hiddenimports = [
    'pygubu.plugins.tk.tkstdwidgets',
    'pygubu.plugins.ttk.ttkstdwidgets',
    'pygubu.plugins.pygubu.dialog',
    'pygubu.plugins.pygubu.editabletreeview',
    'pygubu.plugins.pygubu.scrollbarhelper',
    'pygubu.plugins.pygubu.scrolledframe',
    'pygubu.plugins.pygubu.tkscrollbarhelper',
    'pygubu.plugins.pygubu.tkscrolledframe',
    'pygubu.plugins.pygubu.pathchooserinput',
]
